@@ -1,11 +1,16 @@
 #!/usr/bin/env python3
 # -*- coding: utf-8 -*-
+# PYTHON_ARGCOMPLETE_OK
 import argparse
 import configparser
+import os
+from pathlib import Path
+import shutil
 import sys
 import time
 from typing import Any
 
+import argcomplete
 import requests
 
 import cr_api_client.core_api_client as core_api_client
@@ -113,23 +118,36 @@ def basebox_list_handler(args: Any) -> None:
         )
 
 
+#
+# 'websites_list' related functions
+#
+def websites_list_handler(args: Any) -> None:
+    """List available websites, for use in simulations."""
+    print("[+] List of available websites")
+    websites = core_api_client.fetch_websites()
+
+    for website in websites:
+        print("  [+] {}".format(website))
+
+
 #
 # 'simu_create' simulation related functions
 #
 def simu_create_handler(args: Any) -> None:
-    """Process YAML configuration file and request core API to create a
-    new simulation.
+    """Process YAML topology file and a resource folder and request core API to create a new
+    simulation.
 
     """
 
     # Parameters
-    architecture_file = args.architecture_file
+    topology_file = args.topology_file
+    topology_resources_path = args.topology_resources_path
 
     # Compute elpased time
     t1 = time.time()
 
     try:
-        id_simulation = core_helper.simu_create(architecture_file)
+        id_simulation = core_helper.simu_create(topology_file, topology_resources_path)
         print("Created simulation ID: '{}'".format(id_simulation))
     except Exception as e:
         print(f"Error when creating new simulation: '{e}'")
@@ -144,7 +162,7 @@ def simu_create_handler(args: Any) -> None:
 # 'provisioning_execute' related functions
 #
 def provisioning_execute_handler(args: Any) -> None:
-    """Process YAML configuration file and execute a new provisioning
+    """Process YAML topology file and execute a new provisioning
     chronology (generate + play)).
 
     """
@@ -281,8 +299,8 @@ def simu_status_handler(args: Any) -> None:
 
     for simulation in simulations:
         if (
-            requested_simulation_id is None
-            or requested_simulation_id == simulation["id"]
+                requested_simulation_id is None
+                or requested_simulation_id == simulation["id"]
         ):
             id_simulation = simulation["id"]
 
@@ -340,6 +358,20 @@ def simu_status_handler(args: Any) -> None:
                         )
                     )
 
+                # Display network information
+                network_interfaces = core_api_client.fetch_network_interfaces(
+                    machine["id"]
+                )
+                print("      [+] network:")
+                for network_interface in network_interfaces:
+                    print(
+                        "        [+] IP address: {} (at runtime: {}), MAC address: {}".format(
+                            network_interface["ip_address"],
+                            network_interface["ip_address_runtime"],
+                            network_interface["mac_address"],
+                        )
+                    )
+
 
 #
 # 'simu_pause' simulation
@@ -480,7 +512,7 @@ def simu_delete_handler(args: Any) -> None:
 def simu_snap_handler(args: Any) -> None:
     # Parameters
     id_simulation = args.id_simulation
-    # core generates an architecture file (yaml)
+    # core generates a YAML topology file
     # that will be located to /cyber_range_stuff...
     # This outfile is merely a copy of the generated
     # file
@@ -490,12 +522,143 @@ def simu_snap_handler(args: Any) -> None:
         yaml = core_helper.simu_snap(id_simulation)
         with open(output_file, "w") as w:
             w.write(yaml)
-        print(f"[+] Snapshot done. Architecture file stored at {output_file}")
+        print(f"[+] Snapshot done. Topology file stored at {output_file}")
     except Exception as e:
         print(f"Error when creating snapshot for simulation: '{e}'")
         sys.exit(1)
 
 
+#
+# 'topo_add_websites' handler
+#
+def topo_add_websites_handler(args: Any) -> None:
+    """Process YAML topology file and add docker websites node."""
+
+    # Parameters
+    input_topology_file = args.input_topology_file
+    output_topology_file = args.output_topology_file
+    websites = args.websites
+    switch_name = args.switch_name
+
+    try:
+        topology_yaml = core_helper.topology_add_websites(
+            input_topology_file, websites, switch_name
+        )
+        with open(output_topology_file, "w") as w:
+            w.write(topology_yaml)
+        print(f"[+] Topology updated. New topology stored at '{output_topology_file}'")
+    except Exception as e:
+        print(f"Error when adding websites to a topology: '{e}'")
+        sys.exit(1)
+
+
+#
+# 'topo_add_dga' handler
+#
+def topo_add_dga_handler(args: Any) -> None:
+    """Process YAML topology file and add dga on a docker node."""
+
+    # Parameters
+    input_topology_file = args.input_topology_file
+    output_topology_file = args.output_topology_file
+    resources_dir = args.resources_dir
+    algorithm = args.algorithm
+    switch_name = args.switch_name
+    number = args.number
+
+    try:
+        # get the absolute path
+        resources_dir = os.path.abspath(resources_dir)
+
+        # Verify that the destination folder does not exist
+        proceed = True
+        if os.path.exists(resources_dir):
+            print(f"{resources_dir} already exists. It will be replaced.")
+            while (reply := input("Continue? [y/n]").lower()) not in {"y", "n"}:
+                print(f"{resources_dir} already exists. It will be replaced.")
+            if reply == 'y':
+                proceed = True
+            if reply == 'n':
+                proceed = False
+
+            if not proceed:
+                print(f"Stopping.")
+                sys.exit(1)
+            else:
+                # remove resources_dir
+                try:
+                    if os.path.isfile(resources_dir):
+                        os.remove(resources_dir)
+                    else:
+                        shutil.rmtree(resources_dir)
+                except OSError as e:
+                    print("Error: %s - %s." % (e.filename, e.strerror))
+
+        (topology_yaml, domains) = core_helper.topology_add_dga(
+            input_topology_file, algorithm, switch_name, number, resources_dir
+        )
+        # write the topology
+        with open(output_topology_file, "w") as w:
+            w.write(topology_yaml)
+        print(f"[+] Topology updated. New topology stored at '{output_topology_file}'")
+
+        # write the resources (empty html file by default)
+        resources_path = os.path.join(os.getcwd(), resources_dir)
+        os.mkdir(resources_path)
+
+        websites_path = os.path.join(resources_path, "websites")
+        os.mkdir(websites_path)
+
+        nginx_conf_path = os.path.join(resources_path, "nginx-conf.d")
+        os.mkdir(nginx_conf_path)
+
+        # Create empty html file for each website
+        for d in domains:
+            os.mkdir(f"{websites_path}/{d}")
+            Path(f"{websites_path}/{d}/index.html").touch()
+
+        # write default nginx conf
+        nginx_default = """
+        server {
+            listen       80;
+            listen  [::]:80;
+            server_name  localhost;
+            location / {
+                root   /usr/share/nginx/html;
+                index  index.html index.htm;
+            }
+            error_page   500 502 503 504  /50x.html;
+            location = /50x.html {
+                root   /usr/share/nginx/html;
+            }
+        }
+        """
+        with open(f"{nginx_conf_path}/default.conf", "w") as default_conf:
+            default_conf.write(nginx_default)
+
+        # write local nginx conf
+        template_generated_domains = """
+                 server {{
+                     listen      80;
+                     listen      443;
+                     server_name {0};
+                     location / {{
+                         root    /usr/share/nginx/html/{0};
+                     }}
+                 }}
+                 """
+        with open(f"{nginx_conf_path}/local-websites.conf", "a") as nginx_conf:
+            for d in domains:
+                nginx_conf.write(template_generated_domains.format(d))
+
+        print(f"[+] Resources created. They are stored at '{output_topology_file}'")
+        print(f"[+] Domains added: '{domains}'")
+
+    except Exception as e:
+        print(f"Error when adding domains to a topology: '{e}'")
+        sys.exit(1)
+
+
 def set_core_api_url(core_api_url: str) -> str:
     print("  [+] Using core API URL: {}".format(core_api_url))
     core_api_client.CORE_API_URL = core_api_url
@@ -548,7 +711,6 @@ def set_key(key: str) -> str:
 
 
 def main() -> None:
-
     print("[+] Config")
     parser = argparse.ArgumentParser()
 
@@ -630,6 +792,13 @@ def main() -> None:
     )
     parser_bb_list.set_defaults(func=basebox_list_handler)
 
+    # 'websites_list' command
+    parser_websites_list = subparsers.add_parser(
+        "websites_list",
+        help="List available websites",
+    )
+    parser_websites_list.set_defaults(func=websites_list_handler)
+
     # -----------------------
     # --- Core/simu options
     # -----------------------
@@ -644,8 +813,15 @@ def main() -> None:
         "-a",
         action="store",
         required=True,
-        dest="architecture_file",
-        help="Input path of simulation architecture",
+        dest="topology_file",
+        help="Input path of a YAML topology file",
+    )
+    parser_simu_create.add_argument(
+        "-r",
+        action="store",
+        required=False,
+        dest="topology_resources_path",
+        help="Input path of simulation topology resources",
     )
 
     # 'simu_run' simulation command
@@ -715,8 +891,7 @@ def main() -> None:
         "--output",
         type=str,
         required=True,
-        default=None,
-        help="Path to the output yaml file",
+        help="Path to the output YAML topology file",
     )
 
     # 'simu_clone' simulation command
@@ -750,6 +925,100 @@ def main() -> None:
     parser_simu_untap.add_argument("id_simulation", type=int, help="The simulation id")
     parser_simu_untap.add_argument("iface", type=str, help="The tap network interface")
 
+    # --------------------
+    # --- Topology options
+    # --------------------
+
+    # 'topo_add_websites' command
+    parser_topo_add_websites = subparsers.add_parser(
+        "topo_add_websites", help="Add docker websites node to a topology"
+    )
+    parser_topo_add_websites.set_defaults(func=topo_add_websites_handler)
+    parser_topo_add_websites.add_argument(
+        "-i",
+        "--input",
+        type=str,
+        required=True,
+        dest="input_topology_file",
+        help="Path to the input YAML topology file",
+    )
+    parser_topo_add_websites.add_argument(
+        "-o",
+        "--output",
+        type=str,
+        required=True,
+        dest="output_topology_file",
+        help="Path to the output YAML topology file that will be created",
+    )
+    parser_topo_add_websites.add_argument(
+        "-s" "--switch",
+        type=str,
+        required=True,
+        dest="switch_name",
+        help="Switch name on which to add a docker node",
+    )
+    parser_topo_add_websites.add_argument(
+        "-w" "--websites",
+        type=str,
+        required=True,
+        nargs="+",
+        dest="websites",
+        help="List of websites to add, taken from the websites catalog",
+    )
+    # 'topo_add_dga' command
+    parser_topo_add_dga = subparsers.add_parser(
+        "topo_add_dga", help="Add docker websites node to a topology using a domain generation algorithm"
+    )
+    parser_topo_add_dga.set_defaults(func=topo_add_dga_handler)
+    parser_topo_add_dga.add_argument(
+        "-i",
+        "--input",
+        type=str,
+        required=True,
+        dest="input_topology_file",
+        help="Path to the input YAML topology file",
+    )
+    parser_topo_add_dga.add_argument(
+        "-o",
+        "--output",
+        type=str,
+        required=True,
+        dest="output_topology_file",
+        help="Path to the output YAML topology file that will be created",
+    )
+    parser_topo_add_dga.add_argument(
+        "-s",
+        "--switch",
+        type=str,
+        required=True,
+        dest="switch_name",
+        help="Switch name on which to add a docker node",
+    )
+    parser_topo_add_dga.add_argument(
+        "-a",
+        "--algorithm",
+        type=str,
+        required=True,
+        dest="algorithm",
+        help="Algorithm to choose for the generation of the domains",
+    )
+    parser_topo_add_dga.add_argument(
+        "-n",
+        "--number",
+        type=int,
+        required=True,
+        dest="number",
+        help="Number of domains to generate",
+    )
+    parser_topo_add_dga.add_argument(
+        "-r",
+        "--resources_dir",
+        type=str,
+        required=True,
+        dest="resources_dir",
+        help="Directory to write the resources",
+    )
+
     # -----------------------
     # --- Provisioning options
     # -----------------------
@@ -810,6 +1079,22 @@ def main() -> None:
         help="Machine name used to filter targets",
     )
 
+    # 'provisioning_websites' command
+    parser_provisioning_execute = subparsers.add_parser(
+        "provisioning_execute", help="Execute provisioning chronology for a simulation"
+    )
+    parser_provisioning_execute.set_defaults(func=provisioning_execute_handler)
+    parser_provisioning_execute.add_argument(
+        "id_simulation", type=int, help="The simulation id"
+    )
+    parser_provisioning_execute.add_argument(
+        "-c",
+        action="store",
+        required=True,
+        dest="provisioning_configuration_file",
+        help="Input path of provisioning configuration",
+    )
+
     # -----------------------
     # --- Scenario options
     # -----------------------
@@ -865,6 +1150,8 @@ def main() -> None:
         "id_simulation", type=int, help="The simulation id"
     )
 
+    argcomplete.autocomplete(parser)
+
     if len(sys.argv) == 1:
         parser.print_help(sys.stderr)
         sys.exit(0)
