@@ -149,6 +149,14 @@ def execute_scenario(attacks: list) -> bool:
         return scheduler(attacks)
 
 
+def list_actions() -> str:
+    response = __get("/attack", headers={}, data={})
+    return response.json()
+
+def list_workers() -> str:
+    response = __get("/worker", headers={}, data={})
+    return response.json()
+
 def scheduler(attacks: list) -> bool:
     for attack in attacks:
         time.sleep(5)
