@@ -58,6 +58,19 @@ def __delete(route: str, **kwargs: str) -> Any:
     )
 
 
+def __handle_error(result: requests.Response, context_error_msg: str) -> None:
+    if result.headers.get("content-type") == "application/json":
+        error_msg = result.json()["message"]
+    else:
+        error_msg = result.text
+
+    raise Exception(
+        f"{context_error_msg}. "
+        f"Status code: '{result.status_code}'. "
+        f"Error message: '{error_msg}'."
+    )
+
+
 # -------------------------------------------------------------------------- #
 
 
@@ -82,62 +95,30 @@ def scenario_play(
 ) -> None:
     """Play scenario on targeted simulation."""
 
-    if scenario_path is None:
-        scenario_path = "test/scenarios/scenario0.yml"
-
-    python_api = False
-
-    if os.path.isdir(scenario_path):
-        # scenarios written with Python syntax
-        python_api = True
-    else:
-        # scenarios written with YAML syntax
-        with open(scenario_path, "r") as fd:
-            scenario_yaml = fd.read()
-
-    # Play scenario
-
     scenario_success = False
     scenario_log_filename = None
 
     try:
-        if python_api:
-            data = {
-                "idSimulation": id_simulation,
-                "debug_mode": debug_mode,
-                "speed": speed,
-            }
+        data = {
+            "idSimulation": id_simulation,
+            "debug_mode": debug_mode,
+            "speed": speed,
+        }
 
-            with TemporaryDirectory() as temp_dir:
-                # Zipping scenario files
-                zip_file_name = _zip_scenario(scenario_path, temp_dir)
-                scenario_files = open(zip_file_name, "rb")
-                files = {"scenario_files": scenario_files}
-                try:
-                    result = __post(
-                        "/scenario/start_scenario_py", data=data, files=files
-                    )
-                finally:
-                    scenario_files.close()
-
-        else:
-            data = json.dumps(
-                {"idSimulation": id_simulation, "scenarioYAML": scenario_yaml}
-            )
-
-            result = __post(
-                "/scenario/start_scenario",
-                data=data,
-                headers={"Content-Type": "application/json"},
-            )
+        with TemporaryDirectory() as temp_dir:
+            # Zipping scenario files
+            zip_file_name = _zip_scenario(scenario_path, temp_dir)
+            scenario_files = open(zip_file_name, "rb")
+            files = {"scenario_files": scenario_files}
+            try:
+                result = __post(
+                    "/scenario/start_scenario", data=data, files=files
+                )
+            finally:
+                scenario_files.close()
 
         if result.status_code != 200:
-            raise Exception(
-                "Cannot start scenario at scenario API. "
-                "Status code: '{}'. Error message: '{}'".format(
-                    result.status_code, result.json()["message"]
-                )
-            )
+            __handle_error(result, "Cannot start scenario at scenario API")
 
         # Wait for the operation to be completed in backend
         current_status = ""
@@ -223,12 +204,7 @@ def scenario_status(id_simulation: int) -> None:
         )
 
         if result.status_code != 200:
-            raise Exception(
-                "Cannot get scenario status from scenario API. "
-                "Status code: '{}'. Error message: '{}'".format(
-                    result.status_code, result.json()["message"]
-                )
-            )
+            __handle_error(result, "Cannot get scenario status from scenario API. ")
 
         return result.json()
 
@@ -248,12 +224,7 @@ def scenario_result(id_simulation: int) -> str:
         )
 
         if result.status_code != 200:
-            raise Exception(
-                "Cannot get scenario result from scenario API. "
-                "Status code: '{}'. Error message: '{}'".format(
-                    result.status_code, result.json()["message"]
-                )
-            )
+            __handle_error(result, "Cannot get scenario result from scenario API")
 
         return result.json()
 
